/*
m_sb_loop - Loop one sample from a samplebank

DESCRIPTION
Playback a looped sample from a samplebank with pitch control and control over
the portion of the sample to play.

SYNTAX
aout1, aout2 m_sb_loop Sname, kPitch [, iMode] [, iPosition] [, iStart] \
	[, iEnd]

INITIALIZATION
Sname - Name of the samplebank, as given to m_sb_new
iMode - Looping mode: 0=forwards, 1=backwards 2=back-and-forth (default=0)
iPosition - Sample index to play within the bank (between 0 and size -1)
	(defaults to the next sample within the sequence)
iStart - Start time to play back
	in seconds (default=0)
iEnd - Ending time of the sample slice to play in seconds (default=end)

PERFORMANCE
aout1, aout2 - Audio output
kPitch - Pitch factor or playback speed, 1=normal, <1=slower, >1=faster

CREDITS
Author: Jeanette C.
*/

opcode m_sb_loop, aa, Skojoj
	Sname, kPitch, iMode, iPosition, iStart, iEnd xin

	; Set up the basic channel name
	SchannelName strcat "m_sb.", Sname

	; Audio outputs (there could be stereo samples)
	aOut1 init 0
	aOut2 init 0

	if chnget(strcat(SchannelName, ".Valid")) == 1 then
		; Retrieve some information about the sample bank
		kCurrent chnget strcat(SchannelName, ".Current")
		iSize chnget strcat(SchannelName, ".Size")
		iFirst chnget strcat(SchannelName, ".Start")

		; Find which sample to actually play, next in sequence or user supplied
		if iPosition == -1 then
			if kCurrent >= iSize then ; wrap round to start
				iIndex = 0
			else
				iIndex = i(kCurrent) + 1
			endif
		else ; A position was supplied
			if iPosition >= iSize then ; too big, set to last sample
				iIndex = (iSize - 1)
			else ; it's find
				iIndex = iPosition
			endif
		endif

		; Set the current position to the new value
		chnset iIndex, strcat(SchannelName, ".Current")

		; Get information on the sample to play
		iLength = nsamp(iFirst + iIndex)
		iSampleRate = ftsr(iFirst + iIndex)
		iTotalTime = iLength / iSampleRate
		iChnls = ftchnls(iFirst + iIndex)

		; Check start and end times
		if iStart >= iTotalTime then
			prints "\nWARNING m_sb_loop %s: start time too big, setting to 0", Sname
			iRealStart = 0
		else
			iRealStart = iStart
		endif

		if iEnd == -1 then
			iRealEnd = iTotalTime
		elseif iEnd > iTotalTime then
			prints "\nWARNING m_sb_loop %s: End time too big, setting to sample length.", Sname
			iRealEnd = iTotalTime
		else
			iRealEnd = iEnd
		endif

		iDuration = iRealEnd - iRealStart

		if iChnls == 1 then
			aOut1 flooper2 1, kPitch, iRealStart, iDuration, 0, (iFirst + iIndex), \
				iRealStart, iMode, 0, 0
			aOut2 = aOut1
		else ; it's a stereo sample
			aOut1, aOut2 flooper2 1, kPitch, iRealStart, iDuration, 0, \
				(iFirst + iIndex), iRealStart, iMode, 0, 0
		endif
	else ; This samplebank is not valid
		prints "\nERROR m_sb_loop %s: Tis samplebank does not exist.", Sname
		aOut1 = 0
		aOut2 = 0
	endif

	; Output the audio
	xout aOut1, aOut2
endop
