/*
m_sb_waveseq - Use the samplebank as a wave sequence

DESCRIPTION
Play through the samples, triggering a new one by a give frequency. There is
also pitch control for the sample playback speed. A sample will be played at
most once in its slot. It may be truncated if it is too long to fit its slot.

SYNTAX
aOut1, aOut2 m_sb_waveseq Sname, kPitch, kTriggerFreq [, kTriggerDrift]

INITIALIZATION
Sname - Name of the samplebank, as given to m_sb_new

PERFORMANCE
aOut1, aOut2 - the two audio outputs, if a sample is mono, its channel will
	be copied to the other output too
kPitch - Playback speed of the samples, 1=normal, <1=slower, >1=faster
kTriggerFreq - Frequency at which new samples are triggered
kTriggerDrift - maximum factor the trigger frequency may drift (default=0)

CREDITS
Author: Jeanette C.
*/

opcode m_sb_waveseq, aa, SkkO
	Sname, kPitch, kTriggerFreq, kTriggerDrift xin

	setksmps 1 ; we need the syncphasor and comparison to aSyncOut

	; Set up the basic channel name
	SchannelName strcat "m_sb.", Sname
	
	
	; Initialise the audio outputs
	aOut1 init 0
	aOut2 init 0

	; Initialise the audio signals for the two phasors
	aTrigPhasor init 0
	aTrigSyncIn init 0
	aTrigSyncOut init 0
	aPlayPhasor init 0
	aPlaySyncOut init 0
	kDriftFactor init 1 ; trigger frequency drift

	; Only perform if this samplebank is valid/ready to play
	if chnget(strcat(SchannelName, ".Valid")) == 1 then
		; Generate the amp ftable
		iAmpFn ftgenonce 0, 0, 4096, 7, 1, 4000, 1, 96, 0

		; Get basic samplebank info
		iFirst chnget strcat(SchannelName, ".Start")
		iSize chnget strcat(SchannelName, ".Size")
		iSamplerateFn chnget strcat(SchannelName, ".Samplerates")
		iChannelFn chnget strcat(SchannelName, ".Channels")

		; Variables for the sequencing
		kPosition init 0
		kPlayCount init 0 ; to make it oneshot samples in each slot

		; Run the two phasors
		aTrigPhasor, aTrigSyncOut syncphasor (kTriggerFreq * kDriftFactor), aTrigSyncIn

		; Generate the amp, this will avoid pops if samples are truncated
		kAmp table k(aTrigPhasor), iAmpFn, 1
		
		; Get the current sample and some information
		kSamplerate init tab_i(0,iSamplerateFn)
		kChnls init tab_i(0, iChannelFn, 0)
		kLength init ftlen(iFirst)
		kPlayFreq = kPitch / (kLength / (kSamplerate * kChnls)) ; play phasor frequency
		aPlayPhasor, aPlaySyncOut syncphasor kPlayFreq, aTrigSyncOut

		; Set the play counter (one play through a sample only)
		if k(aPlaySyncOut) == 1 then
			kPlayCount += 1
		endif

		if kPlayCount == 0 then
			if kChnls == 1 then ; Mono
				aOut1 tablekt aPlayPhasor, (iFirst + kPosition), 1, 0, 1
				aOut1 *= kAmp
				aOut2 = aOut1
			else ; stereo
				aReader = aPlayPhasor * (kLength / 2)
				aOut1 tablekt (aReader * 2), (iFirst + kPosition), 0, 0, 1
				aOut2 tablekt (aReader * 2 + 1), (iFirst + kPosition), 0, 0, 1
				aOut1 *= kAmp
				aOut2 *= kAmp
			endif
		else ; Play count is changed
			aOut1 = 0
			aOut2 = 0
		endif

		if k(aTrigSyncOut) == 1 then ; reset play counter and table
			; Set a new trigger frequency factor
			kDriftFactor = 1 + gauss:k(kTriggerDrift)
			kPlayCount = 0
			if (kPosition >= (iSize -1)) then ; set to 0
				kPosition = 0
			else ; just increase
				kPosition += 1
			endif
			kLength tableng (iFirst + kPosition)
			kSamplerate tab kPosition, iSamplerateFn, 0
		endif
	else ; bank not valid
		prints "\nERROR m_sb_waveseq %s: This samplebank is not valid. Was it deleted?", Sname
		aOut1 = 0
		aOut2 = 0
	endif

	; Output the audio
	xout aOut1, aOut2
endop
